/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* This file contains the definitions of types and functions related to the configuration
 and that can be called by extensions. */
 
#ifndef _CONF_API_H
#define _CONF_API_H

#include <stddef.h>
#include <sys/socket.h>
#include <netinet/in.h>

/* The version of this API, to check binary compatibility -- increment each time a change is made in api_conf_t */
#define WAAAD_API_CONF_VER	1

#ifndef sSA_ALIASES
#define sSS	struct sockaddr_storage
#define sSA	struct sockaddr
#define sSA4	struct sockaddr_in
#define sSA6	struct sockaddr_in6
#define sSA_ALIASES
#endif /* sSA_ALIASES */

/* This type contains all the parts of the global configuration of the daemon that can be accessed by the extensions */
typedef struct {

	char		*diameter_identity;	/* The Diameter Identity of the local peer */
	uint16_t	 local_port;		/* The port on which to listen for incoming connexions. */
	
	sSS		 local_addr_diamid[2];	/* The first address resolved from the Diameter Id (IP and IPv6),
						   to use in Host-IP-Address if following list is empty. */
	sSS		*local_addr_pri_sa;	/* points to a 0-terminated array of sSS containing the primary addresses from conf file. */
	sSS		*local_addr_sec_sa;	/* points to a 0-terminated array of sSS containing the secondary addresses from conf file. */
	
	int		 disable_inet4;		/* if != 0, the local host does not use the IP protocol */
	int		 disable_inet6;		/* if != 0, the local host does not use the IPv6 protocol */
	int		 disable_tcp;		/* if != 0, the local host does not use the TCP protocol */
	int		 disable_relay;		/* if != 0, the local host does not relay Diameter messages */
	
	int		 tctimer;		/* The value in seconds of the Tc timer */
	int		 twtimer;		/* The value in seconds of the Tw timer */
	
} ext_conf_t; /* One should use the always-defined "g_pconf" pointer to access these fields */

/* Now define the type of the structure that contains the callback to pass to extensions */
typedef struct {
	/* The header is common to all sub-API pieces */
	size_t	length;		/* The size of this structure, may be useful for extensions not using the conf facility */
	int	version;	/* The version of this API/ABI, must be WAAAD_API_CONF_VER */
	
	/* the remaining is conf-specific */
	ext_conf_t 	*ext_conf; /* Pointer to the ext_conf part of the daemon */
	
} api_conf_t;

#ifdef IN_EXTENSION

/* From within the extensions, we access to conf through this structure */
#ifdef DECLARE_API_POINTERS
api_conf_t * g_api_conf=NULL;
#else /* DECLARE_API_POINTERS */
extern api_conf_t * g_api_conf;
#endif /* DECLARE_API_POINTERS */

/* This define allows to access fields in the same way as in waaad */
#define g_pconf	(g_api_conf->ext_conf)


#else /* IN_EXTENSION */

/* From the daemon, we must initialize the API object, in extension.c */
# define MY_WAAAD_API_CONF_VER 1
# if MY_WAAAD_API_CONF_VER != WAAAD_API_CONF_VER
#  error "You must update API_INIT_CONF also"
# endif

#define API_INIT_CONF( api_conf ) 			\
{							\
	(api_conf).length     = sizeof(api_conf_t);	\
	(api_conf).version    = WAAAD_API_CONF_VER;	\
	(api_conf).ext_conf   = g_pconf;		\
}							

#endif /* IN_EXTENSION */

#endif /* ! _CONF_API_H */
